#include "stdafx.h"
#include "wlan.h"

CWLan::CWLan():
	m_hClient(NULL)
{
	memset(&m_GUID,0,sizeof(m_GUID));
}

CWLan::~CWLan()
{
	close();
}

bool CWLan::open()
{
	close();
	DWORD ret;
	WlanOpenHandle(WLAN_API_MAKE_VERSION(1, 0), NULL, &ret, &m_hClient);
	switch(ret){
	case ERROR_SUCCESS:
		return true;
	case ERROR_INVALID_PARAMETER:
		return false;
	case ERROR_NOT_ENOUGH_MEMORY:
		return false;
	case ERROR_REMOTE_SESSION_LIMIT_EXCEEDED:
		return false;
	default:
		//RPC_STATUS
		//I don't know well...
		//but it seems to work.
		return true;
	}
}

void CWLan::close()
{
	if(m_hClient){
		HANDLE hTmp=m_hClient;
		m_hClient=NULL;
		WlanCloseHandle(hTmp,NULL);
		memset(&m_GUID,0,sizeof(m_GUID));
	}
}

//copies GUID of the first interface
bool CWLan::readGUID()
{
	WLAN_INTERFACE_INFO_LIST *ifList;
	DWORD ret = WlanEnumInterfaces(m_hClient, NULL, &ifList);
	if(ERROR_SUCCESS != ret){
		return false;
	}
	//copy GUID of the first interface
	m_GUID=ifList->InterfaceInfo[0].InterfaceGuid;
	WlanFreeMemory(ifList);

	return true;
}

//makes a list of wifi profiles
bool CWLan::readProfiles()
{
	WLAN_PROFILE_INFO_LIST *profList;
	DWORD ret = WlanGetProfileList(m_hClient, &m_GUID, NULL, &profList);
	if(ERROR_SUCCESS != ret){
		return false;
	}

	//copy
	m_Profiles.assign(profList->ProfileInfo,profList->ProfileInfo + profList->dwNumberOfItems);

	WlanFreeMemory(profList);

	return true;
}

bool CWLan::connect(LPCWSTR lpszProfile)
{
	WLAN_CONNECTION_PARAMETERS connParam;
	connParam.wlanConnectionMode = wlan_connection_mode_profile;
	connParam.strProfile = lpszProfile;
	connParam.pDot11Ssid = NULL;
	connParam.pDesiredBssidList = NULL;
	connParam.dot11BssType = dot11_BSS_type_any;
	connParam.dwFlags = 0;

	DWORD ret = WlanConnect(m_hClient, &m_GUID, &connParam, NULL);
	if(ERROR_SUCCESS != ret){
		//fprintf_s(stderr, "WlanConnect() ERROR [");
		switch(ret){
		case ERROR_INVALID_PARAMETER:
			//fprintf_s(stderr, "ERROR_INVALID_PARAMETER]\n");
			break;
		case ERROR_INVALID_HANDLE:
			//fprintf_s(stderr, "ERROR_INVALID_HANDLE]\n");
			break;
		case ERROR_ACCESS_DENIED:
			//fprintf_s(stderr, "ERROR_ACCESS_DENIED]\n");
			break;
		case 1168:
			//fprintf_s(stderr, "No such profile]\n");
			break;
		default:
			//fprintf_s(stderr, "%d]\n", rv);
			break;
		}
		return false;
	}
	return true;
}

bool CWLan::disconnect()
{
	return ERROR_SUCCESS == WlanDisconnect(m_hClient,&m_GUID,NULL);
}
